# -*- coding: utf-8 -*-

import logging

from django.core.management.base import BaseCommand

import django_gearman_commands.settings
from django_gearman_commands import GearmanServerInfo


log = logging.getLogger(__name__)


class Command(BaseCommand):
    """Pprint overview of Gearman server status and workers."""

    args = '[task_filter]'
    help = 'Print overview of Gearman server status and workers.'

    def handle(self, *args, **options):
        result = ''

        task_filter = ''
        if len(args) > 0:
            task_filter = args[0]

        for server in django_gearman_commands.settings.GEARMAN_SERVERS:
            server_info = GearmanServerInfo(server)
            result += server_info.get_server_info(task_filter)

        self.stdout.write(result)



"""
This code provides a mechanism for running django_gearman_commands internal
test suite without having a full Django project.  It sets up the
global configuration, then dispatches out to `call_command` to
kick off the test suite.

"""

# Setup and configure the minimal settings necessary to
# run the test suite.  Note that Django requires that the
# `DATABASES` value be present and configured in order to
# do anything.

from django.conf import settings
import logging

logging.basicConfig(level=logging.DEBUG, format='%(levelname)s %(asctime)s %(module)s: %(message)s')

settings.configure(
    DEBUG=True,
    
    INSTALLED_APPS=[
        "django_gearman_commands",
    ],
    
    DATABASES={
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
            "NAME": ":memory:",
        }
    },
    
    # tests dependency - run gearman server on localhost on default port
    GEARMAN_SERVERS=[
        'localhost:4730'
    ],
    
    # LocMemCache used as a verification storage for tests
    CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
            'LOCATION': 'django-gearman-commands-tests'
        }
    },

    GEARMAN_CLIENT_NAMESPACE='django-gearman-commands-tests'
)

# Start the test suite now that the settings are configured.
from django.core.management import call_command
call_command("test", "django_gearman_commands")


# -*- coding: utf-8 -*-

import logging
from optparse import make_option
import gearman
from django.core.management.base import BaseCommand, CommandError
import django_gearman_commands.settings
from django_gearman_commands import get_namespace


log = logging.getLogger(__name__)


class Command(BaseCommand):
    """Submit specific gearman job with job data as an arguments."""
    option_list = BaseCommand.option_list + (
    make_option("--foreground",
        action="store_true",
        dest="foreground",
        default=False,
        help="Wait until task is completed. False by default, tasks are sent to background."),
    )

    args = '<task_name> [job_data]'
    help = 'Submit gearman job with specified task, optionally with job data'

    def handle(self, *args, **options):
        try:
            job_data = ''

            if len(args) == 0:
                raise CommandError('At least task name must be provided.')

            task_name = '{0}@{1}'.format(args[0], get_namespace()) if get_namespace() else args[0]
            if len(args) > 1:
                job_data = args[1]

            self.stdout.write('Submitting job: {0:s}, job data: {1:s}.\n'.format(task_name, job_data if job_data else '(empty)'))

            client = gearman.GearmanClient(django_gearman_commands.settings.GEARMAN_SERVERS)
            is_foreground = options.get('foreground', True)
            result = client.submit_job(task_name, job_data,
                wait_until_complete=True if is_foreground else False,
                background=False if is_foreground else True)
            
            self.stdout.write('Job submission done, result: {0:s}.\n'.format(result))
        except:
            log.exception('Error when submitting gearman job')
            raise



from django.conf import settings

# gearman jobs servers
GEARMAN_SERVERS = getattr(settings, 'GEARMAN_SERVERS', ['127.0.0.1:4730'])

# Namespacing support
GEARMAN_CLIENT_NAMESPACE = getattr(settings, 'GEARMAN_CLIENT_NAMESPACE', '')


# -*- coding: utf-8 -*-

import logging

import gearman

from django.core.management import call_command
from django.core.management.base import BaseCommand

import django_gearman_commands.settings

__version__ = '0.2'

log = logging.getLogger(__name__)

class HookedGearmanWorker(gearman.GearmanWorker):
    """GearmanWorker with hooks support."""

    def __init__(self, exit_after_job, host_list=None):
        super(HookedGearmanWorker, self).__init__(host_list=host_list)
        self.exit_after_job = exit_after_job

    def after_job(self):
        return not self.exit_after_job


class GearmanWorkerBaseCommand(BaseCommand):
    """Base command for Gearman workers.

    Subclass this class in your gearman worker commands.

    """
    worker_class = HookedGearmanWorker

    @property
    def task_name(self):
        """Override task_name property in worker to indicate what task should be registered in Gearman."""
        raise NotImplementedError('task_name should be implemented in worker')

    @property
    def exit_after_job(self):
        """Return True if worker should exit after processing job. False by default.

        You do not need to override this in standard case, except in case
        you want to control and terminate worker after processing jobs.
        Used by test worker 'footest'.

        """
        return False

    def do_job(self, job_data):
        """Gearman job execution logic.

        Override this in worker to perform job.

        """
        raise NotImplementedError('do_job() should be implemented in worker')

    def handle(self, *args, **options):
        try:
            worker = self.worker_class(exit_after_job=self.exit_after_job,
                                       host_list=django_gearman_commands.settings.GEARMAN_SERVERS)
            task_name = '{0}@{1}'.format(self.task_name, get_namespace()) if get_namespace() else self.task_name
            log.info('Registering gearman task: %s', self.task_name)
            worker.register_task(task_name, self._invoke_job)
        except Exception:
            log.exception('Problem with registering gearman task')
            raise

        worker.work()

    def _invoke_job(self, worker, job):
        """Invoke gearman job."""
        try:
            self.stdout.write('Invoking gearman job, task: {0:s}.\n'.format(self.task_name))
            result = self.do_job(job.data if job.data else None)

            log.info('Job finished, task: %s, result %s', self.task_name, result)
            self.stdout.write('Job finished, task: {0:s}\n'.format(self.task_name))

            if result is not None:
                self.stdout.write('{0}\n'.format(result))

            return 'OK' if not isinstance(result, basestring) else result
        except Exception:
            log.exception('Error occurred when invoking job, task: %s', self.task_name)
            raise


class GearmanServerInfo():
    """Administration informations about Gearman server.

    See GearmanAdminClient for reference: http://packages.python.org/gearman/admin_client.html

    """

    def __init__(self, host):
        self.host = host
        self.server_version = None
        self.tasks = None
        self.workers = None
        self.ping_time = None
        self.ping_time_str = None

    def get_server_info(self, task_filter=None):
        """Read Gearman server info - status, workers and and version."""
        result = ''

        # Read server status info.
        client = gearman.GearmanAdminClient([self.host])

        self.server_version = client.get_version()
        self.tasks = client.get_status()
        self.workers = client.get_workers()
        self.ping_time = client.ping_server()
        self.ping_time_str = '{0:0.016f}'.format(self.ping_time)

        # if task_filter is set, filter list of tasks and workers by regex pattern task_filter
        if task_filter:
            # filter tasks
            self.tasks = [item for item in self.tasks if task_filter in item['task']]

            # filter workers by registered task name
            self.workers = [item for item in self.workers if item['tasks'] and task_filter in [t for t in item['tasks']]]

        # sort tasks by task name
        self.tasks = sorted(self.tasks, key=lambda item: item['task'])

        # sort registered workers by task name
        self.workers = sorted(self.workers, key=lambda item: item['tasks'])

        # Use prettytable if available, otherwise raw output.
        try:
            from prettytable import PrettyTable
        except ImportError:
            PrettyTable = None

        if PrettyTable is not None:
            # Use PrettyTable for output.
            # server
            table = PrettyTable(['Gearman Server Host', 'Gearman Server Version', 'Ping Response Time'])
            table.add_row([self.host, self.server_version, self.ping_time_str])
            result += '{0:s}.\n\n'.format(table)

            # tasks
            table = PrettyTable(['Task Name', 'Total Workers', 'Running Jobs', 'Queued Jobs'])
            for r in self.tasks:
                table.add_row([r['task'], r['workers'], r['running'], r['queued']])

            result += '{0:s}.\n\n'.format(table)

            # workers
            table = PrettyTable(['Worker IP', 'Registered Tasks', 'Client ID', 'File Descriptor'])
            for r in self.workers:
                if r['tasks']: # ignore workers with no registered task
                    table.add_row([r['ip'], ','.join(r['tasks']), r['client_id'], r['file_descriptor']])

            result += '{0:s}.\n\n'.format(table)

        else:
            # raw output without PrettyTable
            result += 'Gearman Server Host:{0:s}\n'.format(self.host)
            result += 'Gearman Server Version:{0:s}.\n'.format(self.server_version)
            result += 'Gearman Server Ping Response Time:{0:s}.\n'.format(self.ping_time_str)
            result += 'Tasks:\n{0:s}\n'.format(self.tasks)
            result += 'Workers:\n{0:s}\n'.format(self.workers)

        return result


def get_namespace():
    """Namespace to suffix function on a mutialized gearman."""
    return django_gearman_commands.settings.GEARMAN_CLIENT_NAMESPACE


def submit_job(task_name, data='', client=None, **options):
    """Shortcut util for submitting job in standard way.

    We will revert the default behaviour of GermanClient.submit_job, so that
    job are executed in background by default and client doesn't wait for task completion.

    """
    background = options.pop('background', True)
    wait_until_complete = options.pop('wait_until_complete', False)
    client = client or gearman.GearmanClient(django_gearman_commands.settings.GEARMAN_SERVERS)
    task_name = '{0}@{1}'.format(task_name, get_namespace()) if get_namespace() else task_name

    return client.submit_job(task_name, data=data, background=background, wait_until_complete=wait_until_complete,
                             **options)



# -*- coding: utf-8 -*-
import logging
import pickle

from django.core.management import call_command
from django.test import TestCase
from django.core.cache import cache
from django_gearman_commands import GearmanServerInfo

import django_gearman_commands.settings


log = logging.getLogger(__name__)


print 'test'

class GearmanCommandsTest(TestCase):

    def test_server_info(self):
        server_info = GearmanServerInfo(django_gearman_commands.settings.GEARMAN_SERVERS[0])
        server_info.get_server_info()
        self.assertTrue(server_info.server_version.startswith('OK'), 'Unexpected server version string')
        self.assertTrue(type(server_info.tasks) is list, 'Unexpected server tasks type')

        # verify command is callable
        overview = call_command('gearman_server_info')

    def test_server_info_task_filter(self):
        # submit job to queue so we have something shown up in get_server_info()
        call_command('gearman_submit_job', 'footest')

        server_info = GearmanServerInfo(django_gearman_commands.settings.GEARMAN_SERVERS[0])

        server_info.get_server_info("footest")
        self.assertTrue(len(server_info.tasks) > 0, 'Unexpected server_info.tasks')

        server_info.get_server_info("NON_EXISTING!!")
        self.assertTrue(len(server_info.tasks) == 0, 'Unexpected server_info.tasks')

        # let the worker process the job
        call_command('gearman_worker_footest')

    def test_worker_simple(self):
        # submit job
        call_command('gearman_submit_job', 'footest')

        # let the worker process the job
        call_command('gearman_worker_footest')

        # verify job was processed
        self.assertEqual(cache.get('footest'), 'I AM FOO !', 'Unexpected footest worker result')

    def test_worker_task_data_string(self):
        # submit job
        call_command('gearman_submit_job', 'footest', 'DATA')

        # let the worker process the job
        call_command('gearman_worker_footest')

        # verify job was processed and processed task data
        self.assertEqual(cache.get('footest'), 'DATA', 'Unexpected footest worker result (data string)')

    def test_worker_task_data_pickled(self):
        # submit job
        call_command('gearman_submit_job', 'footest', pickle.dumps(u'DATA'))

        # let the worker process the job
        call_command('gearman_worker_footest')

        # verify job was processed and processed task data
        self.assertEqual(pickle.loads(cache.get('footest')), u'DATA', 'Unexpected footest worker result (data pickled)')


# -*- coding: utf-8 -*-

from django.core.cache import cache

from django_gearman_commands import GearmanWorkerBaseCommand


class Command(GearmanWorkerBaseCommand):
    """Gearman worker performing 'footest' job."""
    
    @property
    def task_name(self):
        return 'footest'

    @property
    def exit_after_job(self):
        return True # terminate after job is handled. Do not do this for standard workers !
    
    def do_job(self, job_data):
        # set data to cache
        cache.set('footest', 'I AM FOO !' if not job_data else job_data)
        

    

    



# -*- coding: utf-8 -*-
import os

from setuptools import setup


# Utility function to read the README file.
# Used for the long_description. It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as readme:
        return readme.read()


setup(
    name='django-gearman-commands',
    version='0.4.0',
    description='Django management commands for working with Gearman job server from Django framework',
    long_description=read('README.rst'),
    author=u'Jozef Ševčík',
    author_email='info@codescale.net',
    url='http://www.codescale.net/en/community#django-gearman-commands',
    download_url='http://github.com/CodeScaleInc/django-gearman-commands/tarball/master',
    license='BSD',
    keywords = 'django gearman gearmand jobs queue',
    packages=['django_gearman_commands'],
    include_package_data=True,
    install_requires=['Django', 'prettytable', 'gearman<=2.0.3.beta'],
    platforms='any',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
        'Framework :: Django',
    ],
    dependency_links = [
        'https://github.com/CodeScaleInc/python-gearman/tarball/master#egg=gearman-2.0.3.beta'
    ]
)